package clothing_search_engine.crawler;
/**
 * 
 */

import java.net.*;
import java.io.*;
import java.util.regex.*;
import java.util.*;

/**
 * @author Sara
 *
 * Network Crawler
 * 
 */
public class Crawler {
	
	private Vector urls;
	private Hashtable visited_urls;
	private String range;
	private String url_reg_expression;
	private String domain;
	private PageExtractor page_extractor;
	
	/**
	 * Constructor - initialize data structures and set domain, etc..
	 * 
	 * @param m_domain domain of website
	 * @param m_range regular expression of range of page 
	 * @param m_url_reg_expression regular expression for url's attempting to find 
	 */
	public Crawler(String m_domain, String m_range, String m_url_reg_expression) {

		domain = m_domain;
		range = m_range;
		url_reg_expression = m_url_reg_expression;
		
		urls = new Vector();
		visited_urls = new Hashtable();
		page_extractor = new PageExtractor(range);
	}

	/**
	 * Crawl the page of the current url. Extract url's on that page and then crawl them.
	 * 
	 * @param starting_url url to start crawl from
	 * @param degree number of degrees to crawl
	 * @return return true upon succes
	 */
	public boolean crawl(URL starting_url, int degree) {

		visited_urls.put(starting_url.toString(),new Url(starting_url.toString()));
		
		// Extract the urls for the first page
		String webPage = page_extractor.getPage(starting_url);
		
		this.extractUrls(new Url(starting_url.toString()),webPage,0);

		// Crawl all the pages until no new pages exist or # of degrees passes in has been reached.
		while(!urls.isEmpty()) {
			Url url = (Url)urls.remove(0);
			
			if(visited_urls.containsKey(url.getUrl()) || url.getDegree() >= degree) {
				continue;
			}
			visited_urls.put(url.getUrl(),url);

			try {
				
				if (url.getUrl().indexOf("http") == -1) {
					url.setUrl(domain + url.getUrl());
				}
				
				webPage = page_extractor.getPage(new URL(url.getUrl()));
				this.extractUrls(url,webPage,url.getDegree()+1);
			
			}catch(Exception e) {
				System.out.println("Failure in main: " + e);
			}
		}

		//return false if failure
		return true;
	}
	
	/**
	 * extract the urls on the given page
	 * 
	 * @param from_url url of page that extracting
	 * @param webPage page that is being extracted
	 * @param degree number of levels to crawl
	 */
	private void extractUrls(Url from_url, String webPage, int degree) {

		Matcher m_url_expression = Pattern.compile("<a.*href=\".*" + url_reg_expression + "</a>").matcher(webPage);

		int end = 0;
		
		while (m_url_expression.find()) {
			end = m_url_expression.end();
			
			Url url = Url.parseLink(webPage.substring(m_url_expression.start(),end),degree);
			
			if (url.getUrl().indexOf("http") == -1) {
				url.setUrl(domain + url.getUrl());
			}

			if (!urls.isEmpty() && ((Url)urls.lastElement()).getUrl().equals(url.getUrl())) {
				
				if (!(url.getImage().equals("") || url.getImage().isEmpty())) {
					((Url)urls.lastElement()).setImage(url.getImage());
				}
				
				if (!(url.getName().equals("") || url.getName().isEmpty())) {
					((Url)urls.lastElement()).setName(url.getName());
				}				
			}
			else {
				urls.add(url);
			}

		}
	}
	
	/**
	 * Print all the nodes
	 *
	 */
	public void printNodes() {
		
		Collection all_urls = visited_urls.values();
		
		Iterator iterator = all_urls.iterator();
		
		System.out.println("There are " + visited_urls.size() + " nodes.");
		
		while (iterator.hasNext()) {
			System.out.println(((Url)iterator.next()).getUrl());
		}
	}
	
	/**
	 * main
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		if (args.length < 4) {
			System.out.println("Missing Arguments. Usage: domain url_file/url range url_regular_expression (degree print_name)");
			return;
		}
		
		String domain = args[0];
		String url_file = args[1];
		String range = args[2];
		String regexp = args[3];
		String split_char = args[4];
		int degree = 1;

		Url.split_char = split_char;
		
		if (args.length > 5) {
			degree = Integer.parseInt(args[5]);
		}
		
		System.out.println("The file/url you entered is : " + url_file);
		URL starting_url;

		try {
			
			Crawler crawler = new Crawler(domain,range,regexp);
			
			String url = "";
			
			//accept one url or a file with a list of urls
			if (url_file.indexOf("www.") >= 0) {
				starting_url = new URL(url_file);
				crawler.crawl(starting_url,degree);
			}
			else {
				BufferedReader input = new BufferedReader(new FileReader(url_file));
				url = input.readLine();
				
				while (url != null) {
					starting_url = new URL(url);
					crawler.crawl(starting_url,degree);				
					url = input.readLine();
				}
				input.close();
			}
			
			crawler.printNodes();
			
		}catch(IOException e) {
			System.out.println("Failure in main: " + e);
		}
	}
}
