package clothing_search_engine.crawler;
/**
 * @author Sara
 *
 */
public class Url {

	String url;
	String image;
	String name;
	String url_parameters;
	static String split_char;
	
	//Degree from original seed in the crawl
	int degree;
	
	/**
	 * Constructor
	 * 
	 * @param m_url
	 * @param m_name
	 * @param m_image
	 * @param m_degree
	 */
	public Url(String m_url,String m_name,String m_image, int m_degree) {
		url = m_url;
		name = m_name;
		image = m_image;
		degree = m_degree;
		
		int	split = url.indexOf(split_char);
		
		if (split > 0) {
			url_parameters = url.substring(split);
			url = url.substring(0,split);
		}
	}
	
	/**
	 * Constructor
	 * 
	 * @param m_url
	 */
	public Url(String m_url) {
		this(m_url,"","",0);
	}
	
	/**
	 * Constructor
	 * 
	 * @param m_url
	 * @param m_name
	 */
	public Url(String m_url,String m_name) {
		this(m_url,m_name,"",0);
	}
	
	/**
	 * 
	 * Modifier Methods
	 * 
	 */
	
	/**
	 * Set the url
	 * 
	 * @param m_url
	 */
	public void setUrl(String m_url) {
		url = m_url;
	}
	
	/**
	 * Set the image 
	 * 
	 * @param m_image
	 */
	public void setImage(String m_image) {
		image = m_image;
	}
	
	/**
	 * Set the name
	 * 
	 * @param m_name
	 */
	public void setName(String m_name) {
		name = m_name;
	}
	
	/**
	 * 
	 * Accessor Methods
	 * 
	 */
	
	
	/**
	 * get the url as a string
	 * 
	 * @return String - url
	 */
	public String getUrl() {
		return url;
	}
	
	/**
	 * get the name associated with the url
	 * 
	 * @return String - name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * get the url of the image associated with the url
	 * 
	 * for: <a href=...><img ...></a>
	 * 
	 * @return String - image url
	 */
	public String getImage() {
		return image;
	}
	
	/**
	 * 
	 * get the degree (this is number of times visited)
	 * 
	 * @return int - degree
	 */
	public int getDegree() {
		return degree;
	}
	
	/**
	 * Parse the link to extract url, and name or image  
	 * 
	 * @param link
	 * @param m_degree
	 * @return
	 */
	public static Url parseLink(String link, int m_degree) {
		int end_url = link.indexOf(">");
		
		String url = link.substring(link.indexOf("href=")+5,end_url);
		String image = "";
		
		if (link.indexOf("<",end_url+1) == -1) {
			return null;
		}
		
		String name = link.substring(end_url+1,link.indexOf("<",end_url+1));
		
		// we don't want quotes (")
		if(url.charAt(0) == '"') {
			url = url.substring(1);
		}
		
		if(url.charAt(url.length()-1) == '"') {
			url = url.substring(0,url.length()-1);
		}
		
		link = link.substring(end_url);
		
		// check to see if there is an image
		if (link.indexOf("img") > 0) {
			int image_start = link.indexOf("=") + 1;
			
			image = link.substring(image_start,link.indexOf(" ",image_start));
			
			// we don't want quotes (")
			if(image.charAt(0) == '"') {
				image = image.substring(1);
			}
			
			if(image.charAt(image.length()-1) == '"') {
				image = image.substring(0,image.length()-1);
			}
		}
		return new Url(url,name,image,m_degree);
	}
	
	/**
	 * Print all the url info  
	 */
	public String toString() {
		return "url: " + url + " name: " + name + " image: " + image + "\n";
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		Url.split_char = "&";
		
		//Tests
		Url test = Url.parseLink("<a href=\"/catalog/outfit.jsp?ensembleId=3511&categoryId=0\">View Outfit</a>",0);
		System.out.println(test.toString());
		
		test = Url.parseLink("<a id=test href=\"/catalog/outfit.jsp?ensembleId=3511&categoryId=0\">View Outfit</a>",0);
		System.out.println(test.toString());
		
		test = Url.parseLink("<a href=\"/catalog/outfit.jsp?ensembleId=3511&categoryId=0\"></a>",0);
		System.out.println(test.toString());
		
		//invalid
		test = Url.parseLink("<a href=\"/catalog/outfit.jsp?ensembleId=3511&categoryId=0\">View Outfit",0);
		if (test != null)
			System.out.println("Invalid, should be null: " + test.toString());
		
		//image type
		test = Url.parseLink("<a href=\"/catalog/outfit.jsp?ensembleId=3511&categoryId=0\"><img src=\"http://anntaylor.scene7.com/is/image/anntaylor/AFE3511M?$outfitcross118x222$\" width=\"118\" height=\"222\" border=\"0\" alt=\"View Outfit\"/></a>",0);
		System.out.println(test.toString());
	}

}
