package clothing_search_engine.feature_finder;

import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.TermEnum;
import org.apache.lucene.index.Term;

/**
 * This class will:
 * 
 * 1. Read in the index (constructor).
 * 2. Compute document frequencies and other methods
 * 3. Store results
 * 
 * 
 * @author Sara Stolbach
 */

/** Compute different frequencies for feature analysis */
public class Frequency {

	private String index;
	
	/**
	 * Constructor
	 * 
	 * @param m_index index that frequencies are being computed for
	 */
	public Frequency(String m_index) {
		index = m_index;
	}
	
	/**
	 * 
	 * Currently prints out the document frequency. Will be altered to compute and store 
	 *
	 */
	public void getDocumentFrequency() {
		
		try {
			IndexReader reader = IndexReader.open(index);
		
		TermEnum terms = reader.terms();
	    
	    int count = 0;
	    
	    do {
	    	count++;
	 
	    	if (terms.docFreq() > 50)
	     		System.out.println(terms.docFreq() + ", " + ((Term)terms.term()).text());
	    }while(terms.next());

	    System.out.println("There are " + reader.numDocs() + " documents and " + count + " terms in the corpus.");
	   
	    reader.close();
		} catch(Exception e) {
			System.out.println("Exception in Frequency.getDocumentFrequency(): " + e);
		}


	}

  
  public static void main(String[] args) throws Exception {
    String usage =
      "Usage: java clothing_search_engine.feature_finder.Frequency [-index dir]";
    if (args.length > 0 && ("-h".equals(args[0]) || "-help".equals(args[0]))) {
      System.out.println(usage);
      System.exit(0);
    }

    String index = "index";
    
    for (int i = 0; i < args.length; i++) {
      if ("-index".equals(args[i])) {
        index = args[i+1];
        i++;
      }
    }
    
    Frequency compute = new Frequency(index);
    compute.getDocumentFrequency();
    
  }
    
}
